@@ -13,6 +13,7 @@ from Products.CMFPlone.tests import dummy
 from AccessControl.User import nobody
 from Acquisition import aq_base
 from DateTime import DateTime
+from zExceptions import BadRequest
 
 default_user = PloneTestCase.default_user
 
@@ -21,17 +22,21 @@ class TestMembershipTool(PloneTestCase.PloneTestCase):
 
     def afterSetUp(self):
         self.membership = self.portal.portal_membership
-        self.membership.memberareaCreationFlag = 0
 
-    def testGetPersonalFolder(self):
-        # Should return the .personal folder
+        # Nuke Administators and Reviewers groups added in 2.1a2 migrations
+        # (and any other migrated-in groups) to avoid test confusion
+        self.portal.portal_groups.removeGroups(self.portal.portal_groups.listGroupIds())
+
+    def addMember(self, username, fullname, email, roles, last_login_time):
+        self.membership.addMember(username, 'secret', roles, [])
+        member = self.membership.getMemberById(username)
+        member.setMemberProperties({'fullname': fullname, 'email': email,
+                                    'last_login_time': DateTime(last_login_time),})
+
+    def testNoMorePersonalFolder(self):
+        # .personal folders are history
         personal = getattr(self.folder, self.membership.personal_id, None)
-        self.failIfEqual(personal, None)
-        self.assertEqual(self.membership.getPersonalFolder(default_user), personal)
-
-    def testGetPersonalFolderIfMissing(self):
-        # Should return None as the .personal folder is missing
-        self.folder._delObject(self.membership.personal_id)
+        self.assertEqual(personal, None)
         self.assertEqual(self.membership.getPersonalFolder(default_user), None)
 
     def testGetPersonalFolderIfNoHome(self):
@@ -108,11 +113,10 @@ class TestMembershipTool(PloneTestCase.PloneTestCase):
         self.logout()
         try:
             self.membership.setPassword('geheim')
-        except:
-            # Bl**dy string exceptions
+        except BadRequest:
             import sys; e, v, tb = sys.exc_info(); del tb
-            if str(e) == 'Bad Request' and str(v) == 'Not logged in.':
-                pass    # Test passed
+            if str(v) == 'Not logged in.':
+                pass
             else:
                 raise
 
@@ -120,7 +124,6 @@ class TestMembershipTool(PloneTestCase.PloneTestCase):
         # Password should be changed and user must not change group membership
         group2 = 'g2'
         groups = self.portal.portal_groups
-        groups.groupWorkspacesCreationFlag = 0
         groups.addGroup(group2, None, [], [])
         group = groups.getGroupById(group2)
         group.addMember(default_user)
@@ -134,7 +137,9 @@ class TestMembershipTool(PloneTestCase.PloneTestCase):
         # return None for unknown users,
         self.assertEqual(self.membership.getMemberById('foo'), None)
         # and return None for users defined outside of the portal.
-        self.assertEqual(self.membership.getMemberById(PloneTestCase.portal_owner), None)
+        ##self.assertEqual(self.membership.getMemberById(PloneTestCase.portal_owner), None)
+        # Since CMF 1.5.2 the membershiptool will search "up"
+        self.failIfEqual(self.membership.getMemberById(PloneTestCase.portal_owner), None)
 
     def testGetMemberByIdIsWrapped(self):
         member = self.membership.getMemberById(default_user)
@@ -171,7 +176,7 @@ class TestMembershipTool(PloneTestCase.PloneTestCase):
 
     def testWrapUserWrapsBareUser(self):
         user = self.portal.acl_users.getUserById(default_user)
-        # XXX: GRUF users are wrapped
+        # TODO: GRUF users are wrapped
         self.failUnless(hasattr(user, 'aq_base'))
         user = aq_base(user)
         user = self.membership.wrapUser(user)
@@ -181,7 +186,7 @@ class TestMembershipTool(PloneTestCase.PloneTestCase):
 
     def testWrapUserWrapsWrappedUser(self):
         user = self.portal.acl_users.getUserById(default_user)
-        # XXX: GRUF users are wrapped
+        # TODO: GRUF users are wrapped
         self.failUnless(hasattr(user, 'aq_base'))
         user = self.membership.wrapUser(user)
         self.assertEqual(user.__class__.__name__, 'MemberData')
@@ -232,12 +237,85 @@ class TestMembershipTool(PloneTestCase.PloneTestCase):
         self.failIfEqual(self.membership.getHomeUrl(), None)
         self.assertEqual(self.membership.getHomeUrl('user2'), None)
 
+    def testGetAuthenticatedMemberInfo(self):
+        member = self.membership.getAuthenticatedMember()
+        member.setMemberProperties({'fullname': 'Test user'})
+        info = self.membership.getMemberInfo()
+        self.assertEqual(info['fullname'], 'Test user')
+
+    def testGetMemberInfo(self):
+        self.membership.addMember('user2', 'secret', ['Member'], [],
+                                  properties={'fullname': 'Second user'})
+        info = self.membership.getMemberInfo('user2')
+        self.assertEqual(info['fullname'], 'Second user')
+
+    def testGetCandidateLocalRolesIncludesLocalRolesOnObjectForManager(self):
+        self.folder._addRole('my_test_role')
+        self.folder.manage_setLocalRoles(default_user, ('Manager','Owner'))
+        roles = self.membership.getCandidateLocalRoles(self.folder)
+        self.failUnless('my_test_role' in roles,
+                        'my_test_role not in: %s'%str(roles))
+
+    def testGetCandidateLocalRolesIncludesLocalRolesOnObjectForAssignees(self):
+        self.folder._addRole('my_test_role')
+        self.folder.manage_setLocalRoles(default_user, ('my_test_role','Owner'))
+        roles = self.membership.getCandidateLocalRoles(self.folder)
+        self.failUnless('Owner' in roles)
+        self.failUnless('my_test_role' in roles)
+        self.assertEqual(len(roles), 2)
+
+    def testGetCandidateLocalRolesForManager(self):
+        self.folder._addRole('my_test_role')
+        self.folder.manage_setLocalRoles(default_user, ('Manager','Owner'))
+        roles = self.membership.getCandidateLocalRoles(self.folder)
+        self.failUnless('Manager' in roles)
+        self.failUnless('Owner' in roles)
+        self.failUnless('Reviewer' in roles)
+
+    def testGetCandidateLocalRolesForOwner(self):
+        self.folder._addRole('my_test_role')
+        roles = self.membership.getCandidateLocalRoles(self.folder)
+        self.failUnless('Owner' in roles)
+        self.assertEqual(len(roles), 1)
+
+    def testGetCandidateLocalRolesForAssigned(self):
+        self.folder._addRole('my_test_role')
+        self.folder.manage_setLocalRoles(default_user, ('Reviewer','Owner'))
+        roles = self.membership.getCandidateLocalRoles(self.folder)
+        self.failUnless('Owner' in roles)
+        self.failUnless('Reviewer' in roles)
+        self.assertEqual(len(roles), 2)
+
+    def test_bug4333_delete_user_remove_memberdata(self):
+        # delete user should delete portal_memberdata
+        memberdata = self.portal.portal_memberdata
+        self.setRoles(['Manager'])
+        self.addMember('barney', 'Barney Rubble', 'barney@bedrock.com', ['Member'], '2002-01-01')
+        barney = self.membership.getMemberById('barney')
+        self.failUnlessEqual(barney.email, 'barney@bedrock.com')
+        del barney
+
+        self.membership.deleteMembers(['barney'])
+        md = memberdata._members
+        self.failIf(md.has_key('barney'))
+
+        # There is an _v_ variable that is killed at the end of each request
+        # which stores a temporary version of the member object, this is
+        # a problem in this test.  In fact, this test does not really
+        # demonstrate the bug, which is actually caused by the script not
+        # using the tool.
+        memberdata._v_temps = None
+
+        self.membership.addMember('barney', 'secret', ['Members'], [])
+        barney = self.membership.getMemberById('barney')
+        self.failIfEqual(barney.fullname, 'Barney Rubble')
+        self.failIfEqual(barney.email, 'barney@bedrock.com')
+
 
 class TestCreateMemberarea(PloneTestCase.PloneTestCase):
 
     def afterSetUp(self):
         self.membership = self.portal.portal_membership
-        self.membership.memberareaCreationFlag = 0
         self.membership.addMember('user2', 'secret', ['Member'], [])
 
     def testCreateMemberarea(self):
@@ -246,16 +324,19 @@ class TestCreateMemberarea(PloneTestCase.PloneTestCase):
         self.membership.createMemberarea('user2')
         memberfolder = self.membership.getHomeFolder('user2')
         self.failUnless(memberfolder, 'createMemberarea failed to create memberarea')
+        # member area creation should be on by default
+        self.failUnless(self.membership.getMemberareaCreationFlag())
 
-    def testWrapUserCreatesMemberarea(self):
-        # This test serves to trip us up should this ever change
-        # Also see http://plone.org/collector/1697
-        members = self.membership.getMembersFolder()
-        user = self.portal.acl_users.getUserById('user2')
-        self.membership.memberareaCreationFlag = 1
-        self.membership.wrapUser(user)
-        memberfolder = self.membership.getHomeFolder('user2')
-        self.failUnless(memberfolder, 'wrapUser failed to create memberarea')
+##     def testWrapUserCreatesMemberarea(self):
+##         # This test serves to trip us up should this ever change
+##         # Also see http://plone.org/collector/1697
+##         members = self.membership.getMembersFolder()
+##         user = self.portal.acl_users.getUserById('user2')
+##         self.membership.memberareaCreationFlag = 1
+##         import pdb; pdb.set_trace()
+##         self.membership.wrapUser(user)
+##         memberfolder = self.membership.getHomeFolder('user2')
+##         self.failUnless(memberfolder, 'wrapUser failed to create memberarea')
 
     def testCreatMemberareaUsesCurrentUser(self):
         # Should create a memberarea for user2
@@ -289,19 +370,33 @@ class TestCreateMemberarea(PloneTestCase.PloneTestCase):
         memberfolder = self.membership.getHomeFolder('user2')
         self.failUnless(memberfolder, 'createMemberArea failed to create memberarea')
 
+    def testCreateMemberareaLPF(self):
+        # Should create a Large Plone Folder instead of a normal Folder
+        self.membership.setMemberAreaType('Large Plone Folder')
+        self.membership.createMemberarea('user2')
+        memberfolder = self.membership.getHomeFolder('user2')
+        self.assertEqual(memberfolder.getPortalTypeName(), 'Large Plone Folder')
+
+    def testCreateMemberareaWhenDisabled(self):
+        # Should not create a member area
+        self.membership.setMemberareaCreationFlag()
+        self.failIf(self.membership.getMemberareaCreationFlag())
+        self.membership.createMemberarea('user2')
+        memberfolder = self.membership.getHomeFolder('user2')
+        self.failIf(memberfolder, 'createMemberarea created memberarea despite flag')
+
 
 class TestMemberareaSetup(PloneTestCase.PloneTestCase):
 
     def afterSetUp(self):
         self.membership = self.portal.portal_membership
-        self.membership.memberareaCreationFlag = 0
         self.membership.addMember('user2', 'secret', ['Member'], [])
         self.membership.createMemberarea('user2')
         self.home = self.membership.getHomeFolder('user2')
 
     def testMemberareaIsPloneFolder(self):
         # Memberarea should be a Plone folder
-        self.assertEqual(self.home.meta_type, 'Plone Folder')
+        self.assertEqual(self.home.meta_type, 'ATFolder')
         self.assertEqual(self.home.portal_type, 'Folder')
 
     def testMemberareaIsOwnedByMember(self):
@@ -315,90 +410,38 @@ class TestMemberareaSetup(PloneTestCase.PloneTestCase):
         self.assertEqual(self.home.get_local_roles_for_userid('user2'), ('Owner',))
 
     def testMemberareaHasDescription(self):
-        # Memberarea should have a description
-        self.failUnless(self.home.Description())
+        # Memberarea should have a description - not in 2.1 ~limi
+        #self.failUnless(self.home.Description())
+        pass
 
     def testMemberareaIsCataloged(self):
         # Memberarea should be cataloged
         catalog = self.portal.portal_catalog
-        self.failUnless(catalog(id='user2', Type='Folder', Title="user2's Home"),
+        self.failUnless(catalog(id='user2', Type='Folder', Title="user2"),
                         "Could not find user2's home folder in the catalog")
 
-    def testHomePageExists(self):
-        # Should have an index_html document
-        self.failUnless(hasattr(aq_base(self.home), 'index_html'))
-
-    def testHomePageIsDocument(self):
-        # Home page should be a Document
-        page = self.home.index_html
-        self.assertEqual(page.meta_type, 'Document')
-        self.assertEqual(page.portal_type, 'Document')
-
-    def testHomePageIsOwnedByMember(self):
-        # Home page should be owned by member
-        page = self.home.index_html
-        try: owner_info = page.getOwnerTuple()
-        except AttributeError:
-            owner_info = page.getOwner(info=1)
-        self.assertEqual(owner_info[0], [PloneTestCase.portal_name, 'acl_users'])
-        self.assertEqual(owner_info[1], 'user2')
-        self.assertEqual(len(page.get_local_roles()), 1)
-        self.assertEqual(page.get_local_roles_for_userid('user2'), ('Owner',))
-
-    def testHomePageHasEmptyDescription(self):
-        # Home page should not have a description (since 2.0.1)
-        page = self.home.index_html
-        self.failIf(page.Description())
-
-    def testHomePageIsCataloged(self):
-        # Home page should be cataloged
-        catalog = self.portal.portal_catalog
-        self.failUnless(catalog(id='index_html', Title="user2's Home Page"))
-
-    def testPersonalFolderExists(self):
-        # Should have a .personal folder
-        self.failUnless(hasattr(aq_base(self.home), self.membership.personal_id))
-
-    def testPersonalFolderIsPloneFolder(self):
-        # .personal should be a Plone folder
-        personal = self.home[self.membership.personal_id]
-        self.assertEqual(personal.meta_type, 'Plone Folder')
-        self.assertEqual(personal.portal_type, 'Folder')
-
-    def testPersonalFolderIsOwnedByMember(self):
-        # .personal should be owned by member
-        personal = self.home[self.membership.personal_id]
-        try: owner_info = personal.getOwnerTuple()
-        except AttributeError:
-            owner_info = personal.getOwner(info=1)
-        self.assertEqual(owner_info[0], [PloneTestCase.portal_name, 'acl_users'])
-        self.assertEqual(owner_info[1], 'user2')
-        self.assertEqual(len(personal.get_local_roles()), 1)
-        self.assertEqual(personal.get_local_roles_for_userid('user2'), ('Owner',))
-
-    def testPersonalFolderHasDescription(self):
-        # .personal should have a description
-        personal = self.home[self.membership.personal_id]
-        self.failUnless(personal.Description())
-
-    def testPersonalFolderIsNotCataloged(self):
-        # .personal should not be cataloged
-        catalog = self.portal.portal_catalog
-        self.failIf(catalog(Title='Personal Items'))
+    def testHomePageNotExists(self):
+        # Should not have an index_html document anymore
+        self.failIf('index_html' in self.home.objectIds())
 
 
 class TestSearchForMembers(PloneTestCase.PloneTestCase):
 
     def afterSetUp(self):
-        self.membership = self.portal.portal_membership
-        self.membership.memberareaCreationFlag = 0
         self.memberdata = self.portal.portal_memberdata
+        self.membership = self.portal.portal_membership
         # Don't let default_user disturb results
         self.portal.acl_users._doDelUsers([default_user])
         # Add some members
-        self.addMember('fred', 'Fred Flintstone', 'fred@bedrock.com', ['Member', 'Reviewer'], '2002-01-01')
-        self.addMember('barney', 'Barney Rubble', 'barney@bedrock.com', ['Member'], '2002-01-01')
-        self.addMember('bambam', 'Bambam Rubble', 'bambam@bambam.net', ['Member'], '2003-12-31')
+        self.addMember('fred', 'Fred Flintstone',
+                       'fred@bedrock.com', ['Member', 'Reviewer'],
+                       '2002-01-01')
+        self.addMember('barney', 'Barney Rubble',
+                       'barney@bedrock.com', ['Member'],
+                       '2002-01-01')
+        self.addMember('brubble', 'Bambam Rubble',
+                       'bambam@bambam.net', ['Member'],
+                       '2003-12-31')
         # MUST reset this
         self.memberdata._v_temps = None
 
@@ -408,7 +451,15 @@ class TestSearchForMembers(PloneTestCase.PloneTestCase):
         member.setMemberProperties({'fullname': fullname, 'email': email,
                                     'last_login_time': DateTime(last_login_time),})
 
+    def testSearchById(self):
+        # Should search id and fullname
+        search = self.membership.searchForMembers
+        self.assertEqual(len(search(name='brubble')), 1)
+        self.assertEqual(len(search(name='barney')), 1)
+        self.assertEqual(len(search(name='rubble')), 2)
+
     def testSearchByName(self):
+        # Should search id and fullname
         search = self.membership.searchForMembers
         self.assertEqual(len(search(name='rubble')), 2)
         self.assertEqual(len(search(name='stone')), 1)
@@ -428,6 +479,13 @@ class TestSearchForMembers(PloneTestCase.PloneTestCase):
         self.assertEqual(len(search(last_login_time=DateTime('2002-01-01'))), 3)
         self.assertEqual(len(search(last_login_time=DateTime('2003-01-01'))), 1)
 
+    def testSearchLoginBeforeSpecifiedTime(self):
+        search = self.membership.searchForMembers
+        self.assertEqual(len(search(last_login_time=DateTime('2002-01-02'),
+                                    before_specified_time=True)), 2)
+        self.assertEqual(len(search(last_login_time=DateTime('2004-01-01'),
+                                    before_specified_time=True)), 3)
+
     def testSearchByNameAndEmail(self):
         search = self.membership.searchForMembers
         self.assertEqual(len(search(name='rubble', email='bedrock')), 1)
@@ -443,6 +501,18 @@ class TestSearchForMembers(PloneTestCase.PloneTestCase):
         self.assertEqual(len(search(name='rubble', last_login_time=DateTime('2002-01-01'))), 2)
         self.assertEqual(len(search(name='flintstone', last_login_time=DateTime('2003-01-01'))), 0)
 
+    def testSearchByNameAndLoginBeforeSpecifiedTime(self):
+        search = self.membership.searchForMembers
+        self.assertEqual(len(search(name='rubble',
+                                    last_login_time=DateTime('2002-01-01'),
+                                    before_specified_time=True)), 0)
+        self.assertEqual(len(search(name='rubble',
+                                    last_login_time=DateTime('2002-01-02'),
+                                    before_specified_time=True)), 1)
+        self.assertEqual(len(search(name='flintstone',
+                                    last_login_time=DateTime('2003-01-01'),
+                                    before_specified_time=True)), 1)
+
     def testSearchByEmailAndRoles(self):
         search = self.membership.searchForMembers
         self.assertEqual(len(search(email='fred', roles=['Reviewer'])), 1)
@@ -453,12 +523,38 @@ class TestSearchForMembers(PloneTestCase.PloneTestCase):
         self.assertEqual(len(search(email='bedrock', last_login_time=DateTime('2002-01-01'))), 2)
         self.assertEqual(len(search(email='bedrock', last_login_time=DateTime('2003-01-01'))), 0)
 
+    def testSearchByEmailAndLoginBeforeSpecifiedTime(self):
+        search = self.membership.searchForMembers
+        self.assertEqual(len(search(email='bedrock',
+                                    last_login_time=DateTime('2002-01-01'),
+                                    before_specified_time=True)), 0)
+        self.assertEqual(len(search(email='bedrock',
+                                    last_login_time=DateTime('2003-01-01'),
+                                    before_specified_time=True)), 2)
+
     def testSearchByRolesAndLastLoginTime(self):
         search = self.membership.searchForMembers
         self.assertEqual(len(search(roles=['Member'], last_login_time=DateTime('2002-01-01'))), 3)
         self.assertEqual(len(search(roles=['Reviewer'], last_login_time=DateTime('2002-01-01'))), 1)
         self.assertEqual(len(search(roles=['Member'], last_login_time=DateTime('2003-01-01'))), 1)
 
+    def testSearchByRolesAndLoginBeforeSpecifiedTime(self):
+        search = self.membership.searchForMembers
+        self.assertEqual(len(search(roles=['Member'],
+                                    last_login_time=DateTime('2002-01-01'),
+                                    before_specified_time=True)), 0)
+        self.assertEqual(len(search(roles=['Member'],
+                                    last_login_time=DateTime('2003-01-01'),
+                                    before_specified_time=True)), 2)
+        self.assertEqual(len(search(roles=['Member'],
+                                    last_login_time=DateTime('2004-01-01'),
+                                    before_specified_time=True)), 3)
+        self.assertEqual(len(search(roles=['Reviewer'],
+                                    last_login_time=DateTime('2002-01-01'),
+                                    before_specified_time=True)), 0)
+        self.assertEqual(len(search(roles=['Reviewer'],
+                                    last_login_time=DateTime('2004-01-01'),
+                                    before_specified_time=True)), 1)
 
 def test_suite():
     from unittest import TestSuite, makeSuite
